"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.LiveCoordinator = exports.LiveMonitor = exports.LiveProvider = exports.LiveSource = undefined;

var _callbackToAsyncIterator = require("./callbackToAsyncIterator");

var _callbackToAsyncIterator2 = _interopRequireDefault(_callbackToAsyncIterator);

var _lodash = require("lodash");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable flowtype/no-weak-types */
class LiveSource {
  subscribeCollection(_callback, _collectionIdentifier, _predicate) {
    return null;
  }

  subscribeRecord(_callback, _collectionIdentifier, _recordIdentifier) {
    return null;
  }
}

exports.LiveSource = LiveSource;
class LiveProvider {

  constructor(namespace) {
    this.namespace = namespace;
    this.sources = [];
  }

  registerSource(source) {
    this.sources.push(source);
  }

  collectionIdentifierIsValid(_collectionIdentifier) {
    return false;
  }

  recordIdentifierIsValid(_collectionIdentifier, _recordIdentifier) {
    return false;
  }
}

exports.LiveProvider = LiveProvider;
class LiveMonitor {

  constructor(providers) {
    this.providers = providers;
    this.subscriptionReleasers = [];
    this.handleChange = (0, _lodash.debounce)(this.handleChange.bind(this), 250, {
      leading: true,
      trailing: true
    });
    this.onChange = this.onChange.bind(this);
  }

  reset() {
    // clear monitoring
    for (const releaser of this.subscriptionReleasers) {
      releaser();
    }
    this.subscriptionReleasers = [];
  }

  release() {
    this.reset();
  }

  handleChange() {
    if (this.changeCallback) {
      this.reset();
      this.changeCallback();
    } else {
      // eslint-disable-next-line no-console
      console.warn("Change occurred, but no-one was listening");
    }
  }

  onChange(callback) {
    if (this.changeCallback) {
      throw new Error("Already monitoring for changes");
    }
    // Debounce to every 250ms
    this.changeCallback = callback;
    setImmediate(this.handleChange);
    return () => {
      if (this.changeCallback === callback) {
        this.changeCallback = null;
      }
    };
  }

  liveCollection(namespace, collectionIdentifier, predicate = () => true) {
    const provider = this.providers[namespace];
    if (!provider || provider.sources.length === 0) return;
    if (!provider.collectionIdentifierIsValid(collectionIdentifier)) {
      throw new Error(`Invalid collection identifier passed to LiveMonitor[${namespace}]: ${collectionIdentifier}`);
    }
    for (const source of provider.sources) {
      const releaser = source.subscribeCollection(this.handleChange, collectionIdentifier, predicate);
      if (releaser) {
        this.subscriptionReleasers.push(releaser);
      }
    }
  }

  liveRecord(namespace, collectionIdentifier, recordIdentifier) {
    // TODO: if (recordIdentifier == null) {return}
    const provider = this.providers[namespace];
    if (!provider || provider.sources.length === 0) return;
    if (!provider.collectionIdentifierIsValid(collectionIdentifier)) {
      throw new Error(`Invalid collection identifier passed to LiveMonitor[${namespace}]: ${collectionIdentifier}`);
    }
    if (!provider.recordIdentifierIsValid(collectionIdentifier, recordIdentifier)) {
      throw new Error(`Invalid record identifier passed to LiveMonitor[${namespace}]: ${collectionIdentifier}`);
    }
    for (const source of provider.sources) {
      const releaser = source.subscribeRecord(this.handleChange, collectionIdentifier, recordIdentifier);
      if (releaser) {
        this.subscriptionReleasers.push(releaser);
      }
    }
  }
}

exports.LiveMonitor = LiveMonitor;
class LiveCoordinator {

  constructor() {
    this.providers = {};
    this.subscribe = this.subscribe.bind(this);
  }

  registerProvider(provider) {
    const { namespace } = provider;
    if (this.providers[namespace]) {
      throw new Error(`Namespace ${namespace} already registered with Live`);
    }
    this.providers[namespace] = provider;
  }

  registerSource(namespace, source) {
    if (!this.providers[namespace]) {
      // eslint-disable-next-line no-console
      console.warn(`LiveProvider '${namespace}' is not registered, skipping live source.`);
      return;
    }
    this.providers[namespace].registerSource(source);
  }

  getMonitorAndContext() {
    const monitor = new LiveMonitor(this.providers);
    return {
      monitor,
      context: {
        liveCollection: monitor.liveCollection.bind(monitor),
        liveRecord: monitor.liveRecord.bind(monitor),
        liveConditions: []
      }
    };
  }

  subscribe(_parent, _args, context, _info) {
    const { monitor, context: additionalContext } = this.getMonitorAndContext();
    Object.assign(context, additionalContext);
    return makeAsyncIteratorFromMonitor(monitor);
  }
}

exports.LiveCoordinator = LiveCoordinator;
function makeAsyncIteratorFromMonitor(monitor) {
  return (0, _callbackToAsyncIterator2.default)(monitor.onChange, {
    onClose: release => {
      release();
    }
  });
}
//# sourceMappingURL=Live.js.map